// WtBase.js
//
// Copyright  2005, glenn andreas.	 All rights reserved
//

// common mixins
mixinBroadcaster = Mixin({
	_init : function() {
		this.notifications = {};
	},
	notify : function(name, param) {
		var listeners = this.notifications[name];
		for (var i=0;i<listeners.length;i++) {
			var recv = listeners[i].listener;
			var routine = listeners[i].routine;
			var context = listeners[i].context;
			routine.apply(recv, param, context);
		}
	},
	addListener: function(name, recv, routine, context) {
		if (!this.notification[name]) {
			this.notification[name] = [];
		}
		this.notifications[name].push({'listener' : recv, 'routine': routine, 'context': context});
	},
	removeListener : function(recv) {
		for (var i in this.notifications) {
			var n = this.notifications[i];
			for (var j = 0; j<n.length;j++) {
				if (n[j].listener == recv) {
					n.splice(j,j+1); // remove it
					j--;
				}
			}
		}
	}
});

mixinCursorTracking = Mixin({
	mouseEnter : function(event) {
		this.notify('mouseEnter',event);
	},
	mouseExit : function(event) {
		this.notify('mouseExit', event);
	},
	mouseMove : function(event) {
		this.notify('mouseMove',event);
	},
	installMouseTracking : function() {
		var self = this;
		this.element.onmouseover = function(event) { self.mouseMove(event); };
		this.element.onmouseexit = function(event) { self.mouseExit(event); };
		this.element.onmouseenter = function(event) { self.mouseEnter(event); };
	}
});

mixinHitTracking = Mixin({
	mouseDown : function(event) {
	},
	mouseUp : function(event) {
	},
	mouseMove : function(event) {
	}
});




protoImgBackground = View({
	_proto : protoImage,
	viewStyle : {
		position: 'absolute',
		top: '0px',
		left: '0px'
	}
})


protoPushButton = View({
	_proto: protoView,
	defaultTitle: "Button",
	defaultWidth: 60,
	ViewSetupChildrenScript : function() {
		protoView.ViewSetupChildrenScript.call(this);
		var t = this;
		createGenericButton(this.viewElement,this.defaultTitle,function() { t.ButtonClickScript(); },this.defaultWidth);
	}
});


// for a 'stretchable' view border
protoSlicedStretchBorder = View({
	_proto : protoView,
	sliceSize : [0,0, 0,0],
	sliceBase : 'Base',
	viewClass : "stretchBorder",
	ViewSetupChildrenScript : function() {
		protoView.ViewSetupChildrenScript.call(this);
		this.stepChildren.push(View({
			_proto : protoImage,
			viewJustify : vjParentLeftH | vjParentTopV,
			viewBounds : RelBounds(0,0, this.sliceSize[0], this.sliceSize[2]),
			imageSrc : this.sliceBase + "_top_left.png"
		}));
		this.stepChildren.push(View({
			_proto : protoImage,
			viewJustify : vjParentFullH | vjParentTopV,
			viewBounds : SetBounds(this.sliceSize[0],0, this.sliceSize[1], this.sliceSize[2]),
			imageSrc : this.sliceBase + "_top_center.png"
		}));
		this.stepChildren.push(View({
			_proto : protoImage,
			viewJustify : vjParentRightH | vjParentTopV,
			viewBounds : RelBounds(-this.sliceSize[1],0, this.sliceSize[1], this.sliceSize[2]),
			imageSrc : this.sliceBase + "_top_right.png"
		}));

		this.stepChildren.push(View({
			_proto : protoImage,
			viewJustify : vjParentLeftH | vjParentFullV,
			viewBounds : SetBounds(0,this.sliceSize[2], this.sliceSize[0], this.sliceSize[3]),
			imageSrc : this.sliceBase + "_middle_left.png"
		}));
		this.stepChildren.push(View({
			_proto : protoImage,
			viewJustify : vjParentFullH | vjParentFullV,
			viewBounds : SetBounds(this.sliceSize[0],this.sliceSize[2], this.sliceSize[1], this.sliceSize[3]),
			imageSrc : this.sliceBase + "_middle_center.png"
		}));
		this.stepChildren.push(View({
			_proto : protoImage,
			viewJustify : vjParentRightH | vjParentFullV,
			viewBounds : SetBounds(-this.sliceSize[1],this.sliceSize[2], 0, this.sliceSize[3]),
			imageSrc : this.sliceBase + "_middle_right.png"
		}));

		this.stepChildren.push(View({
			_proto : protoImage,
			viewJustify : vjParentLeftH | vjParentBottomV,
			viewBounds : RelBounds(0, -this.sliceSize[3], this.sliceSize[0], this.sliceSize[3]),
			imageSrc : this.sliceBase + "_bottom_left.png"
		}));
		this.stepChildren.push(View({
			_proto : protoImage,
			viewJustify : vjParentFullH | vjParentBottomV,
			viewBounds : SetBounds(this.sliceSize[0],-this.sliceSize[3], this.sliceSize[1], 0),
			imageSrc : this.sliceBase + "_bottom_center.png"
		}));
		this.stepChildren.push(View({
			_proto : protoImage,
			viewJustify : vjParentRightH | vjParentBottomV,
			viewBounds : RelBounds(-this.sliceSize[1],-this.sliceSize[3], this.sliceSize[1], this.sliceSize[3]),
			imageSrc : this.sliceBase + "_bottom_right.png"
		}));
	},
	ReorientToScreen : function() {
		protoView.ReorientToScreen.call(this);
		if (this.stepChildren.length) {
			// change image widths to stretch them
			var width = this.viewElement.offsetWidth - this.sliceSize[0] - this.sliceSize[1];
			this.stepChildren[1].viewElement.width = width;
			this.stepChildren[4].viewElement.width = width;
			this.stepChildren[6].viewElement.width = width;
			var height = this.viewElement.offsetHeight - this.sliceSize[2] - this.sliceSize[3];
			this.stepChildren[3].viewElement.height = height;
			this.stepChildren[4].viewElement.height = height;
			this.stepChildren[5].viewElement.height = height;
		}
	}
});
